#
# spec file for package hypre
#
# Copyright (c) 2019 SUSE LINUX GmbH, Nuernberg, Germany.
#
# All modifications and additions to the file contributed by third parties
# remain the property of their copyright owners, unless otherwise agreed
# upon. The license for this file, and modifications and additions to the
# file, is the same license as for the pristine package itself (unless the
# license for the pristine package is not an Open Source License, in which
# case the license is the MIT License). An "Open Source License" is a
# license that conforms to the Open Source Definition (Version 1.9)
# published by the Open Source Initiative.

# Please submit bugfixes or comments via https://bugs.opensuse.org/
#


%global flavor mvapich2
%define ver 2.15.1
%define _ver 2_15_1
%define somver 0
%define sover %{somver}.2.6

# Base package name
%define pname hypre
%define PNAME %(echo %{pname} | tr [a-z] [A-Z])

%if 0%{?is_opensuse} || 0%{?is_backports}
%undefine DisOMPI3
%else
%define DisOMPI3 ExclusiveArch:  do_not_build
%endif

%if "%flavor" == ""
%define package_name %{pname}
ExclusiveArch:  do_not_build
%endif

%if "%{flavor}" == "serial"
%undefine suffix
%undefine mpi_family
%bcond_with hpc
%define install_doc 1
%endif

%if "%{flavor}" == "openmpi"
%global mpi_family  openmpi
%define mpi_vers 1
%bcond_with hpc
%endif

%if "%{flavor}" == "openmpi2"
%global mpi_family  openmpi
%define mpi_vers 2
%bcond_with hpc
%endif

%if "%{flavor}" == "mvapich2"
%global mpi_family  mvapich2
%bcond_with hpc
%endif

%if "%{flavor}" == "mpich"
%global mpi_family  mpich
%bcond_with hpc
%endif

%if "%{flavor}" == "gnu-mvapich2-hpc"
%undefine c_f_ver
%define mpi_family mvapich2
%define compiler_family gnu
%bcond_without hpc
%endif

%if "%{flavor}" == "gnu-mpich-hpc"
%undefine c_f_ver
%define mpi_family mpich
%define compiler_family gnu
%bcond_without hpc
%endif

%if "%{flavor}" == "gnu-openmpi-hpc"
%undefine c_f_ver
%define compiler_family gnu
%define mpi_family openmpi
%define mpi_vers 1
%bcond_without hpc
%endif

%if "%{flavor}" == "gnu-openmpi2-hpc"
%undefine c_f_ver
%define compiler_family gnu
%define mpi_family openmpi
%define mpi_vers 2
%bcond_without hpc
%endif

%if "%{flavor}" == "gnu-openmpi3-hpc"
%undefine c_f_ver
%define compiler_family gnu
%define mpi_family openmpi
%define mpi_vers 3
%bcond_without hpc
%{?DisOMPI3}
%endif

# Don't build non-HPC on SLE
%if !0%{?is_opensuse} && !0%{?with_hpc:1}
ExclusiveArch:  do_not_build
%endif

%{?mpi_family:%{bcond_without mpi}}%{!?mpi_family:%{bcond_with mpi}}
%{?with_hpc:%{!?compiler_family:%global compiler_family gnu}}

%{?with_mpi:%global my_suffix -%{mpi_family}}

%if %{without hpc}
 %if %{without mpi}
  %define my_prefix %_prefix
  %define my_bindir %_bindir
  %define my_libdir %_libdir
  %define my_incdir %_includedir
 %else
  %global my_prefix %{_libdir}/mpi/gcc/%{mpi_family}
  %global my_bindir %{my_prefix}/bin
  %global my_libdir %{my_prefix}/%{_lib}/
  %global my_incdir %{my_prefix}/include/
  %global my_datadir %{my_prefix}/share/
 %endif
 %if 0%{!?package_name:1}
  %define package_name   %pname%{?my_suffix}
 %endif
 %define libname  lib%{PNAME}%{somver}%{?my_suffix}
%else # with hpc
%{hpc_init -c %compiler_family %{?c_f_ver:-v %{c_f_ver}} %{?with_mpi:-m {%mpi_family}} %{?mpi_ver:-V %{mpi_ver}} %{?ext:-e %{ext}}}
 %define my_prefix %hpc_prefix
 %define my_bindir %hpc_bindir
 %define my_libdir %hpc_libdir
 %define my_incdir %hpc_includedir
 %define package_name   %{hpc_package_name %_ver}
 %define libname lib%{PNAME}%{hpc_package_name_tail %{?_ver}}
%endif

Name:           %package_name
Version:        %{ver}
Release:        1.1
Summary:        Scalable algorithms for solving linear systems of equations
License:        LGPL-2.1
Group:          Development/Tools/Other
Url:            https://www.llnl.gov/casc/hypre/
Source:         https://github.com/LLNL/hypre/archive/v%{version}.tar.gz#/hypre-%{version}.tar.gz
# TODO : add babel
#BuildRequires:  babel-devel
%if %{without hpc}
BuildRequires:  gcc-c++
BuildRequires:  gcc-fortran
BuildRequires:  lapack-devel
BuildRequires:  superlu-devel
%if 0%{?with mpi}
BuildRequires:  %{mpi_family}-devel
%endif
%else
Requires:       %{compiler_family}%{?c_f_ver}-compilers-hpc
Requires:       superlu%{?hpc_ext}-%{compiler_family}%{?c_f_ver}-hpc-devel
BuildRequires:  %{compiler_family}%{?c_f_ver}-compilers-hpc-macros-devel
BuildRequires:  %{mpi_family}%{?mpi_vers}-%{compiler_family}%{?c_f_ver}-hpc-macros-devel
BuildRequires:  libopenblas%{?hpc_ext}-%{compiler_family}%{?c_f_ver}-hpc-devel
BuildRequires:  superlu%{?hpc_ext}-%{compiler_family}%{?c_f_ver}-hpc-devel
BuildRequires:  suse-hpc
%hpc_requires
%hpc_requires_devel
%endif
BuildRequires:  fdupes
# Default library install path

%description
Hypre is a library of preconditioners that feature parallel multigrid
methods for both structured and unstructured grid problems arising in
the simulation codes being developed at LLNL to study physical
phenomena in the defense, environmental, energy, and biological
sciences.

%package     -n %{libname}
Summary:        Scalable algorithms for solving linear systems of equations
Group:          System/Libraries

%description -n %{libname}
Hypre is a library of preconditioners that feature parallel multigrid
methods for both structured and unstructured grid problems arising in
the simulation codes being developed at LLNL to study physical
phenomena in the defense, environmental, energy, and biological
sciences.

This package contains %{?mpi_family}%{!?mpi_family:serial} shared libraries of Hypre.

%package devel
Summary:        Headers and library links for %{libname}
Group:          Development/Libraries/C and C++
Requires:       %{libname} = %version

%description devel
%{PNAME} headers and libraries files needed for development
This package contains development files of the %{?mpi_family}%{!?mpi_family:serial} version of Hypre.

%package        doc
Summary:        Development documentation for Hypre
Group:          Documentation/Other

%description    doc
This package contains development documentation for Hypre.

%{?with_hpc:%{hpc_master_package -l -N %PNAME -n lib%{PNAME}%{hpc_package_name_tail}}}
%{?with_hpc:%{hpc_master_package devel}}

%prep
%setup -q -n %{pname}-%{version}
%if %{without hpc}
cat > %{_sourcedir}/baselibs.conf  <<EOF
%{libname}
%{libname}-devel
  requires %{?my_suffix}-<targettype>
  requires "%{libname}-<targettype> = <version>"
EOF
%endif

%build

cd src
%define FLAGS %optflags -fPIC %{?with_hpc:-Dhypre_dgesvd=dgesvd_ -Dhypre_dlamch=dlamch_ -Dhypre_blas_lsame=hypre_lapack_lsame -Dhypre_blas_xerbla=hypre_lapack_xerbla }

export CFLAGS="%{FLAGS}"
export CXXFLAGS="%{FLAGS}"
export LDFLAGS="-pie"

%if %{without hpc}

%{?with_mpi: . %{my_bindir}/mpivars.sh}

%configure \
        --prefix=%{my_prefix} \
        --with-timing \
        --without-openmp \
%if %{with mpi}
        --with-MPI \
        --with-MPI-include=%{my_incdir} \
        --with-MPI-lib-dirs=%{my_libdir} \
%else
        --without-MPI \
%endif
        --with-blas-lib=-lblas \
        --with-lapack \
        --with-lapack-lib="-llapack" \
        --with-mli \
        --with-fei \
        --with-superlu \
        %{?with_mpi: MPI_PREFIX=%%{mpiprefix} }

%else
%hpc_setup

module load openblas
module load superlu
%if %{with mpi}
# set this here for now as 'module purge' sets CC
export CC=mpicc
export CXX=mpicxx
export F77=mpif77
%endif

%hpc_configure \
    --with-timing \
    --with-MPI \
    --with-MPI-include=%{my_incdir} \
    --with-MPI-lib-dirs=%{my_libdir} \
    --without-openmp \
    --with-blas-lib="-L$OPENBLAS_LIB -lopenblas" \
    --with-lapack-lib="-L$OPENBLAS_LIB -lopenblas" \
    --with-mli \
    --with-fei \
    --with-superlu \
    --with-superlu-include=$SUPERLU_INC \
    --with-superlu-lib=$SUPERLU_LIB

%endif

mkdir -p hypre/lib
pushd FEI_mv/femli
make %{?_smp_mflags} all
popd
make %{?_smp_mflags} all
cd ..

%install
%if %{without hpc}
%{?with_mpi: . %{my_bindir}/mpivars.sh}
%else
%{hpc_setup}
%if %{with mpi}
export CXX=mpicxx
%endif
%endif


# %%makeinstall macro does not work with hypre
cd src
make install HYPRE_INSTALL_DIR=%{buildroot}%{my_prefix} \
             HYPRE_LIB_INSTALL=%{buildroot}%{my_libdir} \
             HYPRE_INC_INSTALL=%{buildroot}%{my_incdir}%{!?with_hpc:/%{pname}}
install -m644 hypre/lib/* %{buildroot}%{my_libdir}

# install LLNL FEI headers
mkdir -p %{buildroot}%{my_incdir}/%{!?with_hpc:%{pname}/}FEI_mv
cp -r FEI_mv/fei-base %{buildroot}%{my_incdir}/%{!?with_hpc:%{pname}/}FEI_mv/.
cd ..

# Fix wrong permissions
chmod 644 %{buildroot}%{my_incdir}/%{!?with_hpc:%{pname}/}*.h

# This files are provided with babel - TODO
rm -f %{buildroot}%%{my_libdir}/libsidl*

# shared libraries

pushd %{buildroot}%{my_libdir}
LIBS="$(ls *.a|sed 's|\.a||'|sort)"
mkdir tmp; pushd tmp
for i in $LIBS; do
    if [ "$i" != "libbHYPREClient-F" -a "$i" != "libbHYPREClient-CX" ]
    then
        ar x ../$i.a
        ${CXX:-c++} -shared * -L.. $ADDLIB \
                       -Wl,-soname,$i.so.%{somver} -o ../$i.so.%{sover}
	ln -s $i.so.%sover ../$i.so.%{somver}
        ln -s $i.so.%{somver} ../$i.so
	rm -f *
        ADDLIB="-lHYPRE"
    fi
done
popd
rm -rf tmp
rm libHYPRE.a
popd

%if 0%{?install_doc}
install -d %{buildroot}%{_docdir}/%{package_name}
cp -fR docs/* %{buildroot}%{_docdir}/%{package_name}/
%endif

%if %{with hpc}
%hpc_write_modules_files
#%Module1.0#####################################################################

proc ModulesHelp { } {

puts stderr " "
puts stderr "This module loads the hypre library built with the %{compiler_family} compiler"
puts stderr "toolchain and the %{mpi_family} MPI stack."
puts stderr " "
puts stderr "Note that this build of hypre leverages the superlu and MKL libraries."
puts stderr "Consequently, these packages are loaded automatically with this module."

puts stderr "\nVersion %{version}\n"

}
module-whatis "Name: %{pname} built with %{compiler_family} compiler and %{mpi_family} MPI"
module-whatis "Version: %{version}"
module-whatis "Category: runtime library"
module-whatis "%{url}"

set     version                     %{version}

prepend-path    PATH                %{hpc_bindir}
prepend-path    INCLUDE             %{hpc_includedir}
prepend-path    LD_LIBRARY_PATH     %{hpc_libdir}

setenv          %{PNAME}_DIR        %{hpc_prefix}
setenv          %{PNAME}_BIN        %{hpc_bindir}
setenv          %{PNAME}_INC        %{hpc_includedir}
setenv          %{PNAME}_LIB        %{hpc_libdir}

EOF
%endif
%fdupes -s %{buildroot}%{my_prefix}

%post -n %{libname} -p /sbin/ldconfig
%postun -n %{libname} -p /sbin/ldconfig

%files -n %{libname}
%{?hpc_dirs}
%{?hpc_modules_files}
%{my_libdir}/*.so.*

%files devel
%{?hpc_dirs}
%license COPYING.LESSER COPYRIGHT
%doc CHANGELOG README
%{my_incdir}%{?!with_hpc:/%{pname}}
%{my_libdir}/*.so

%if 0%{?install_doc}
%files doc
%{_docdir}/%{package_name}
%endif

%changelog
* Mon Jan 14 2019 eich@suse.com
- Fix yet another library dependency.
* Mon Jan 14 2019 eich@suse.com
- Fix HPC libname for master package.
* Sun Jan 13 2019 eich@suse.com
- Disable openmpi3 builds on SLE.
* Sat Jan 12 2019 Jan Engelhardt <jengelh@inai.de>
- Trim future goals, trim filler wording.
* Mon Jan  7 2019 eich@suse.com
- Add more MPI flavors.
- Create baselib.conf on the fly.
- Do not build non-HPC on SLE to save build cycles.
- Remove babel_files: not used anywhere.
- Remove unneeded BuildRequires:.
* Mon Jan  7 2019 cgoll@suse.com
- Added HPC support.
* Sun Jan  6 2019 Stefan Brüns <stefan.bruens@rwth-aachen.de>
- Update to 2.15.1
- Drop hypre-2.8.0b-no-date-and-time-fix.patch
- Convert to multibuild for openmpi/openmpi2
* Thu Aug  2 2012 scorot@free.fr
- add baselibs.conf
- build serial version
- fix shared libs package name
* Thu Aug  2 2012 scorot@free.fr
- patch0
- move %%%%doc files to devel package
* Mon Jul 30 2012 scorot@free.fr
- add mvapich2 build for SLE 11
* Sun Jul 29 2012 scorot@free.fr
- fist package based on alt linux spec file
